==> down.php?SDK=iOS_V1.20.2.3.zip: iOS_V1.20.2.3/Sample/Source/IAPV4/IAPV4ViewController.swift <==
//
//  IAPV4ViewController.swift
//  HIVE_SDK_Sample
//
//  Created by jhjang on 25/03/2019.
//  Copyright © 2019 gcp. All rights reserved.
//

import UIKit
import HIVEService
import HIVEBase
import HIVEIAPV4

enum IAPV4Api: String, CaseIterable {
    case marketConnect = "Market Connect"
    case getProductInfo = "Get Product Info"
    case getMarketProductInfo = "Get Market Product Info"
    case purchase = "Purchase"
    case restore = "Restore"
    case getProductInfoSub = "Get Product Info Sub"
    case purchaseSub = "Purchase Sub"
    case restoreSubscription = "RestoreSubscription"
    case transactionFinish = "Transaction Finish"
    case transactionMultiFinish = "Transaction Multi Finish"
    case checkPromotePurchase = "Check Promote Purchase"
    case iapv2marketConnect = "IAPV2 Market Connect"
    case iapv2GetShopInfo = "IAPV2 Get Shop Info"
    case iapv2Purchase = "IAPV2 Purchase"
}

class IAPV4ViewController: BaseViewController {
    var isSub:Bool = false
    let apis: [IAPV4Api] = IAPV4Api.allCases
    var products: [IAPV4Product]?
    var receipts: [IAPV4Receipt] = [IAPV4Receipt]()
    var productActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Product", message: "Select Product", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let products = self.products {
            for product in products {
                let actionStyle:UIAlertAction.Style = .default
                let title = """
                    \(product.title ?? "")(\(product.productDescription ?? "")) \(product.displayPrice ?? "")(\(product.currency ?? ""))
                    """
                let action = UIAlertAction(title: title, style: actionStyle) { (action) in
                    guard
                        let marketPID = product.marketPid
                        else {
                            preconditionFailure("Market PID is empty or nil.")
                    }
                    if (self.isSub) {
                        IAPV4Interface.purchaseSubscriptionUpdate(marketPID, oldMarketPid: nil, additionalInfo: nil) { (result, receipt) in
                            let log = """
                            === IAPV4Interface.purchaseSubscriptionUpdate() ===
                            
                            result - \(result)
                            receipt - \(String(describing: receipt))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.isSuccess() {
                                self.receipts.append(receipt!)
                            }
                        }
                    }
                    else {
                        IAPV4Interface.purchase(marketPID, additionalInfo: nil, handler: { (result, receipt) in
                            let log = """
                            === IAPV4Interface.purchase() ===
                            
                            result - \(result)
                            receipt - \(String(describing: receipt))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.isSuccess() {
                                self.receipts.append(receipt!)
                            }
                        })
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    var receiptActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Receipt", message: "Select Receipt", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        for receipt in self.receipts {
            let actionStyle:UIAlertAction.Style = .default
            let title = """
            \(receipt.product?.title ?? "")(\(receipt.product?.productDescription ?? "")) \(receipt.product?.displayPrice ?? "")(\(receipt.product?.currency ?? ""))
            """
            let action = UIAlertAction(title: title, style: actionStyle) { (action) in
                guard
                    let marketPID = receipt.product?.marketPid
                    else {
                        preconditionFailure("Market PID is empty or nil.")
                }
                
                IAPV4Interface.transactionFinish(marketPID, handler: { (result, marketPID) in
                    let log = """
                    === IAPV4Interface.transactionFinish() ===
                    
                    result - \(result)
                    marketPID - \(String(describing: marketPID))
                    """
                    
                    self.logDelegate?.addLog(log: log)
                    
                    if result.isSuccess() {
                        self.receipts.removeAll { $0.product?.marketPid == marketPID }
                    }
                })
            }
            actionSheet.addAction(action)
        }
        
        return actionSheet
    }
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: IAPV4Api) {
        switch api {
        case .marketConnect:
            IAPV4Interface.marketConnect { (result, marketIDs) in
                let log = """
                === IAPV4Interface.marketConnect() ===
                
                result - \(result)
                marketIDs - \(String(describing: marketIDs))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .getProductInfo:
            IAPV4Interface.getProductInfo { (result, products, balance) in
                let log = """
                === IAPV4Interface.getProductInfo() ===
                
                result - \(result)
                products - \(String(describing: products))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.isSub = false
                self.products = products
            }
        case .getMarketProductInfo:
            let pidList = self.products?.map({ $0.marketPid ?? "" })
            IAPV4Interface.getMarketProductInfo(pidList ?? []) { result, products, balance in
                let log = """
                === IAPV4Interface.getMarketProductInfo() ===
                
                result - \(result)
                products - \(String(describing: products))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .purchase:
            self.present(self.productActionSheet, animated: true, completion: nil)
        case .restore:
            IAPV4Interface.restore { (result, receiptList) in
                let log = """
                === IAPV4Interface.restore() ===
                
                result - \(result)
                receiptList - \(String(describing: receiptList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                if result.isSuccess() {
                    self.receipts = receiptList!
                }
            }
        case .getProductInfoSub:
            IAPV4Interface.getSubscriptionProductInfo { result, productInfoList, balance in
                let log = """
                === IAPV4Interface.getSubscriptionProductInfo() ===
                
                result - \(result)
                            productInfoList - \(String(describing: productInfoList))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.isSub = true
                self.products = productInfoList
            }
        case .purchaseSub:
            self.present(self.productActionSheet, animated: true, completion: nil)
        case .restoreSubscription:
            IAPV4Interface.restoreSubscription { (result, receiptList) in
                let log = """
                === IAPV4Interface.restoreSubscription() ===
                
                result - \(result)
                receiptList - \(String(describing: receiptList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                if result.isSuccess() {
                    self.receipts = receiptList!
                }
            }
        case .transactionFinish:
            self.present(self.receiptActionSheet, animated: true, completion: nil)
        case .checkPromotePurchase:
            IAPV4Interface.checkPromotePurchase { result, marketPID in
                let log = """
                === IAPV4Interface.checkPromotePurchase() ===
                
                result - \(result)
                marketPID - \(String(describing: marketPID))
                """
                self.logDelegate?.addLog(log: log)
            }
        case .iapv2marketConnect:
            IAPInterface.initialize { (result, marketInfos) in
                let log = """
                === IAPInterface.initialize() ===
                
                result - \(result)
                marketList - \(String(describing: marketInfos))
                """
                self.logDelegate?.addLog(log: log)
            }
        case .iapv2GetShopInfo:
            IAPInterface.getShopInfo("main") { (result, shop, balance) in
                let log = """
                === IAPInterface.getShopInfo() ===
                
                result - \(result)
                shop - \(String(describing: shop))
                balance = \(balance)
                """
                self.logDelegate?.addLog(log: log)
                
            }
        case .iapv2Purchase:
            IAPInterface.purchase("1001", additionalInfo: "") { (result, receipt) in
                let log = """
                === IAPInterface.purchase() ===
                
                result - \(result)
                receipt - \(String(describing: receipt))
                """
                self.logDelegate?.addLog(log: log)
            }
        default:
            preconditionFailure("Not supported api")
        }
    }
    
}

extension IAPV4ViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? IAPV4TableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}

==> down.php?SDK=iOS_V1.20.3.2.zip: iOS_V1.20.3.2/Sample/Source/IAPV4/IAPV4ViewController.swift <==
//
//  IAPV4ViewController.swift
//  HIVE_SDK_Sample
//
//  Created by jhjang on 25/03/2019.
//  Copyright © 2019 gcp. All rights reserved.
//

import UIKit
import HIVEService
import HIVEBase
import HIVEIAPV4

// swiftlint:disable type_body_length

enum IAPV4Api: String, CaseIterable {
    case marketConnect = "Market Connect"
    case getProductInfo = "Get Product Info"
    case getMarketProductInfo = "Get Market Product Info"
    case purchase = "Purchase"
    case restore = "Restore"
    case getProductInfoSub = "Get Product Info Sub"
    case purchaseSub = "Purchase Sub"
    case restoreSubscription = "RestoreSubscription"
    case transactionFinish = "Transaction Finish"
    case transactionMultiFinish = "Transaction Multi Finish"
    case checkPromotePurchase = "Check Promote Purchase"
    case iapv2marketConnect = "IAPV2 Market Connect"
    case iapv2GetShopInfo = "IAPV2 Get Shop Info"
    case iapv2Purchase = "IAPV2 Purchase"
}

class IAPV4ViewController: BaseViewController {
    var isSub: Bool = false
    let apis: [IAPV4Api] = IAPV4Api.allCases
    var products: [IAPV4Product]?
    var receipts: [IAPV4Receipt] = [IAPV4Receipt]()
    var productActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Product", message: "Select Product", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let products = self.products {
            for product in products {
                let actionStyle: UIAlertAction.Style = .default
                let title = """
                    \(product.title ?? "")(\(product.productDescription ?? "")) \(product.displayPrice ?? "")(\(product.currency ?? ""))
                    """
                let action = UIAlertAction(title: title, style: actionStyle) { (_) in
                    guard
                        let marketPID = product.marketPid
                        else {
                            preconditionFailure("Market PID is empty or nil.")
                    }
                    if self.isSub {
                        IAPV4Interface.purchaseSubscriptionUpdate(marketPID, oldMarketPid: nil, additionalInfo: nil) { (result, receipt) in
                            let log = """
                            === IAPV4Interface.purchaseSubscriptionUpdate() ===
                            
                            result - \(result)
                            receipt - \(String(describing: receipt))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.isSuccess() {
                                self.receipts.append(receipt!)
                            }
                        }
                    } else {
                        IAPV4Interface.purchase(marketPID, additionalInfo: nil, handler: { (result, receipt) in
                            let log = """
                            === IAPV4Interface.purchase() ===
                            
                            result - \(result)
                            receipt - \(String(describing: receipt))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.isSuccess() {
                                self.receipts.append(receipt!)
                            }
                        })
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    var receiptActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Receipt", message: "Select Receipt", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        for receipt in self.receipts {
            let actionStyle: UIAlertAction.Style = .default
            let title =
            "\(receipt.product?.title ?? "")" +
            "(\(receipt.product?.productDescription ?? ""))" +
            " \(receipt.product?.displayPrice ?? "")" +
            " (\(receipt.product?.currency ?? ""))"
            let action = UIAlertAction(title: title, style: actionStyle) { (_) in
                guard
                    let marketPID = receipt.product?.marketPid
                    else {
                        preconditionFailure("Market PID is empty or nil.")
                }
                
                IAPV4Interface.transactionFinish(marketPID, handler: { (result, marketPID) in
                    let log = """
                    === IAPV4Interface.transactionFinish() ===
                    
                    result - \(result)
                    marketPID - \(String(describing: marketPID))
                    """
                    
                    self.logDelegate?.addLog(log: log)
                    
                    if result.isSuccess() {
                        self.receipts.removeAll { $0.product?.marketPid == marketPID }
                    }
                })
            }
            actionSheet.addAction(action)
        }
        
        return actionSheet
    }
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: IAPV4Api) {
        switch api {
        case .marketConnect:
            IAPV4Interface.marketConnect { (result, marketIDs) in
                let log = """
                === IAPV4Interface.marketConnect() ===
                
                result - \(result)
                marketIDs - \(String(describing: marketIDs))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .getProductInfo:
            IAPV4Interface.getProductInfo { (result, products, balance) in
                let log = """
                === IAPV4Interface.getProductInfo() ===
                
                result - \(result)
                products - \(String(describing: products))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.isSub = false
                self.products = products
            }
        case .getMarketProductInfo:
            let pidList = self.products?.map({ $0.marketPid ?? "" })
            IAPV4Interface.getMarketProductInfo(pidList ?? []) { result, products, balance in
                let log = """
                === IAPV4Interface.getMarketProductInfo() ===
                
                result - \(result)
                products - \(String(describing: products))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .purchase:
            self.present(self.productActionSheet, animated: true, completion: nil)
        case .restore:
            IAPV4Interface.restore { (result, receiptList) in
                let log = """
                === IAPV4Interface.restore() ===
                
                result - \(result)
                receiptList - \(String(describing: receiptList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                if result.isSuccess() {
                    self.receipts = receiptList!
                }
            }
        case .getProductInfoSub:
            IAPV4Interface.getSubscriptionProductInfo { result, productInfoList, balance in
                let log = """
                === IAPV4Interface.getSubscriptionProductInfo() ===
                
                result - \(result)
                            productInfoList - \(String(describing: productInfoList))
                balance - \(balance)
                """
                
                self.logDelegate?.addLog(log: log)
                
                self.isSub = true
                self.products = productInfoList
            }
        case .purchaseSub:
            self.present(self.productActionSheet, animated: true, completion: nil)
        case .restoreSubscription:
            IAPV4Interface.restoreSubscription { (result, receiptList) in
                let log = """
                === IAPV4Interface.restoreSubscription() ===
                
                result - \(result)
                receiptList - \(String(describing: receiptList))
                """
                
                self.logDelegate?.addLog(log: log)
                
                if result.isSuccess() {
                    self.receipts = receiptList!
                }
            }
        case .transactionFinish:
            self.present(self.receiptActionSheet, animated: true, completion: nil)
        case .checkPromotePurchase:
            IAPV4Interface.checkPromotePurchase { result, marketPID in
                let log = """
                === IAPV4Interface.checkPromotePurchase() ===
                
                result - \(result)
                marketPID - \(String(describing: marketPID))
                """
                self.logDelegate?.addLog(log: log)
            }
        case .iapv2marketConnect:
            IAPInterface.initialize { (result, marketInfos) in
                let log = """
                === IAPInterface.initialize() ===
                
                result - \(result)
                marketList - \(String(describing: marketInfos))
                """
                self.logDelegate?.addLog(log: log)
            }
        case .iapv2GetShopInfo:
            IAPInterface.getShopInfo("main") { (result, shop, balance) in
                let log = """
                === IAPInterface.getShopInfo() ===
                
                result - \(result)
                shop - \(String(describing: shop))
                balance = \(balance)
                """
                self.logDelegate?.addLog(log: log)
                
            }
        case .iapv2Purchase:
            IAPInterface.purchase("1001", additionalInfo: "") { (result, receipt) in
                let log = """
                === IAPInterface.purchase() ===
                
                result - \(result)
                receipt - \(String(describing: receipt))
                """
                self.logDelegate?.addLog(log: log)
            }
        default:
            preconditionFailure("Not supported api")
        }
    }
    
}

extension IAPV4ViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? IAPV4TableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
